"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Handlebars = require("handlebars");
const changeCase = require("change-case");
const template = `/**
 * Autogenerated interface, DO NOT MODIFY
 */
/* tslint:disable */

import { {{imports}} } from "graphql";

const {{name}}InputType = new GraphQLInputObjectType({
  fields: {
    {{fields}}
  },
  name: "{{name}}Input"
})

const {{name}}Type = new GraphQLObjectType({
  fields: {
    {{fields}}
  },
  name: "{{name}}"
});

export { {{name}}Type };
`;
class GraphQlBuilder {
    constructor() {
        this.compiledTemplate = Handlebars.compile(template);
        this.mysqlTypes = {
            blob: "string",
            bigint: "string",
            char: "string",
            date: "string",
            datetime: "string",
            decimal: "float",
            double: "float",
            float: "float",
            int: "int",
            longblob: "string",
            longtext: "string",
            mediumtext: "string",
            set: "string",
            smallint: "int",
            text: "string",
            timestamp: "string",
            tinyint: "boolean",
            varchar: "string"
        };
    }
    renderTs(table, tableName) {
        let types = Object.keys(table).map(colName => this.toGraphType(table[colName].type));
        types.push("GraphQLObjectType");
        types.push("GraphQLInputObjectType");
        types = types.filter((item, pos, self) => self.indexOf(item) === pos).sort();
        const imports = types.join(", ");
        const rows = Object.keys(table).map(colName => this.buildTypeRow(table[colName]));
        const fields = rows.join(", \n \t\t");
        const name = changeCase.pascalCase(tableName);
        const t = this.compiledTemplate({ fields, name, imports });
        // console.log(t);
        return t;
    }
    toGraphType(mysql) {
        const s = this.mysqlTypes[mysql];
        switch (s) {
            case "string":
                return "GraphQLString";
            case "float":
                return "GraphQLFloat";
            case "int":
                return "GraphQLInt";
            case "boolean":
                return "GraphQLBoolean";
            default:
                throw "unknown type " + mysql;
        }
    }
    buildTypeRow(col) {
        const graphType = this.toGraphType(col.type);
        return `${col.field}: { type: ${graphType} }`;
    }
}
exports.default = GraphQlBuilder;
//# sourceMappingURL=graphql-builder.js.map