"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class InterfaceBuilder {
    constructor(settings, mysqlTypes) {
        this.settings = settings;
        this.mysqlTypes = mysqlTypes;
    }
    renderTs(tableClass, table) {
        let stringBuilder = this.settings.defaultClassModifier + " " + tableClass.prefixedClassName + " { \n";
        for (const colName in table) {
            stringBuilder += this.buildTypeRow(table[colName]);
        }
        stringBuilder += "}\n";
        return this.getMetaText() + stringBuilder;
    }
    getMetaText() {
        let meta = `/**\n * Autogenerated interface, DO NOT MODIFY\n */\n`;
        meta += "/* tslint:disable */\n";
        return meta;
    }
    buildTypeRow(col) {
        const tabs = "\t";
        const optional = this.settings.optionalParameters ? "?" : "";
        const tsType = this.getTsType(col.type);
        const field = col.field;
        return `${tabs}"${field}"${optional}: ${tsType};\n`;
    }
    getTsType(type) {
        let ts = this.mysqlTypes[type];
        if (!ts) {
            // tslint:disable-next-line:no-console
            console.error("Unknown type " + type);
            ts = "any";
        }
        return ts;
    }
}
exports.default = InterfaceBuilder;
//# sourceMappingURL=interface-builder.js.map